@@ -3,8 +3,6 @@
 use strict;
 use warnings;
 
-use lib qw(..);
-
 use Log::Log4perl qw(:easy);
 Log::Log4perl->easy_init($DEBUG);
 
@@ -16,6 +14,7 @@ $logger->debug("logging initialized\n");
 
 my $sqs = Amazon::API::SQS->new(
   debug                      => 1,
+  url                        => 'http://localhost:4566',
   raise_serialization_errors => 1,
   logger                     => Log::Log4perl->get_logger(),
 );
@@ -26,7 +25,7 @@ my $sqs = Amazon::API::SQS->new(
 #my @sqs_attributes = Amazon::API::param_n( { Attribute => $attributes } );
 #my @sqs_tags       = Amazon::API::param_n( { Tag       => $tags } );
 
-my $rsp = $sqs->ListQueues;
+my $rsp = $sqs->ListQueues();
 
 print {*STDOUT} Dumper( [ 'response', $rsp ] );
 
