@@ -61,7 +61,7 @@ Readonly::Scalar our $REQUEST_CLASS_TEMPLATE =>
 use Amazon::API::Constants     qw( :all );
 use Amazon::API::Botocore::Pod qw( pod );
 
-our $VERSION = '2.0.11'; ## no critic (RequireInterpolationOfMetachars)
+our $VERSION = '2.0.12'; ## no critic (RequireInterpolationOfMetachars)
 
 our %BOTO_SERVICES;
 
@@ -998,7 +998,7 @@ sub help {
   return pod
     if !@args && !$options{service};
 
-  my $service = $options{'service'};
+  my $service = $options{service};
 
   my $module;
 
@@ -1020,6 +1020,7 @@ sub help {
     }
     else {
       my $description = get_service_descriptions( lc $options{service} );
+
       my $operations
         = $description->[0]->{ lc $options{service} }->{operations};
 
@@ -1070,15 +1071,14 @@ HELP
 
   my $service_name = create_module_name($service);
 
-  my $class = sprintf 'Amazon::API::Botocore::Shape::%s::%s',
-    $service_name, $module;
-
-  print {*STDOUT} "$class\n";
-
   my $file;
+
   # try -s service or
-  for (qw($service_name $service)) {
+  for ( $service_name, $service ) {
+    my $class = sprintf 'Amazon::API::Botocore::Shape::%s::%s', $_, $module;
+
     $file = require_class($class);
+
     last if $file;
   }
 
@@ -1194,7 +1194,7 @@ our @API_METHODS = qw(
 @actions@
 );
 
-our $VERSION = '2.0.11';
+our $VERSION = '2.0.12';
 
 sub new {
   my ( $class, @options ) = @_;
