@@ -23,6 +23,11 @@ our @EXPORT_OK = qw(
   check_type
   flatten
   get_service_from_class
+  is_array
+  is_code
+  is_hash
+  is_regexp
+  is_scalar
   param_n
   require_class
   require_shape
@@ -34,26 +39,41 @@ our %EXPORT_TAGS = ( all => [@EXPORT_OK] );
 Readonly::Scalar our $SHAPE_NAME_TEMPLATE =>
   'Amazon::API::Botocore::Shape::%s::%s';
 
-our $VERSION = '2.0.11'; ## no critic (RequireInterpolationOfMetachars)
+our $VERSION = '2.0.12'; ## no critic (RequireInterpolationOfMetachars)
 
 our $TEMPLATE_START = tell DATA;
 
+sub is_scalar {
+  my ($obj) = @_;
+
+  return ref $obj ? $FALSE : $TRUE;
+}
+
+########################################################################
+sub is_hash   { return _is_type( @_, 'HASH' ); }
+sub is_array  { return _is_type( @_, 'ARRAY' ); }
+sub is_code   { return _is_type( @_, 'CODE' ); }
+sub is_regexp { return _is_type( @_, 'REGEXP' ); }
+########################################################################
+
+########################################################################
+sub _is_type {
+########################################################################
+  my ( $obj, $type ) = @_;
+
+  return ref $obj && reftype($obj) eq $type ? $TRUE : $FALSE;
+}
+
 ########################################################################
 sub create_module_name {
 ########################################################################
   my ($service) = @_;
 
-  my $module_name;
+  return uc $service
+    if length $service == 3; # SQS SNS IAM STS SES RDS...
 
-  if ( length $service == 3 ) {
-    $module_name = uc $service;  # SQS SNS IAM STS SES RDS...
-  }
-  else {
-    # Route53, Cloudwatch? SecretsManager? ...well then provide your own damn name!
-    $module_name = ucfirst $service;
-  }
-
-  return $module_name;
+  # just return name if mix of upper/lower, else upper case first letter
+  return $service =~ /[[:upper:]]/xsm ? $service : ucfirst $service;
 }
 
 ########################################################################
@@ -109,7 +129,7 @@ sub snake_case {
 ########################################################################
   my ($name) = @_;
 
-  while ( $name =~ s/([[:upper:]])([[:lower:]])/lc("_$1").$2/xsme ) { };  # snake_case the CamelCase
+  while ( $name =~ s/([[:upper:]])([[:lower:]])/lc("_$1").$2/xsme ) { }; # snake_case the CamelCase
 
   $name =~ s/^_//xsm;
 
@@ -251,7 +271,7 @@ sub param_n {
 ########################################################################
   my ( $message, $prefix, $idx ) = @_;
 
-  if ( !defined $idx ) {  # first call, check args
+  if ( !defined $idx ) { # first call, check args
     croak 'message argument must be reference'
       if !ref $message;
   }
@@ -427,15 +447,24 @@ our $SHAPE_DEFINITION = decode_json($SHAPE);
 ########################################################################
 sub new {
 ########################################################################
-  my ( $class, $value ) = @_;
-
+  my ($class,  @args ) = @_;
+  
   my $options = $SHAPE_DEFINITION;
   my $type = $options->{type};
 
-  # may have to initialize other types (list?) as well
-  if ( ! $value && $type eq 'structure' ) {
+  my $value;
+  
+  # We may have to initialize other types (list?) as well although I
+  # think this is only valid for a Botocore request object. We want to
+  # avoid calling _init_value in the parent class during construction
+  # if the value is undefined which it will be when we instantiate a
+  # shape when creating a response object
+  if ( @args && ! $args[0] && $type eq 'structure' ) {
     $value = {};
   }
+  elsif (@args) {
+    $value = $args[0];
+  }
   
   $options->{_value} = $value;
   $options->{service} = '@service@';
