@@ -14,6 +14,8 @@ our %EXPORT_TAGS = (
     qw{
       $TRUE
       $FALSE
+      $SUCCESS
+      $FAILURE
     }
   ],
 
@@ -43,7 +45,7 @@ our %EXPORT_TAGS = (
   ],
 );
 
-our $VERSION = '2.0.11';
+our $VERSION = '2.0.12';
 
 # chars
 Readonly::Scalar our $AMPERSAND          => q{&};
@@ -69,6 +71,10 @@ Readonly::Scalar our $PADDING => $SPACE x 4;
 Readonly::Scalar our $TRUE  => 1;
 Readonly::Scalar our $FALSE => 0;
 
+# shell booleans
+Readonly::Scalar our $SUCCESS => 0;
+Readonly::Scalar our $FAILURE => 1;
+
 foreach my $k ( keys %EXPORT_TAGS ) {
   push @EXPORT_OK, @{ $EXPORT_TAGS{$k} };
 }
